# Copyright 2018-2024 contributors to the OpenLineage project
# SPDX-License-Identifier: Apache-2.0

import time
from unittest.mock import patch

from openlineage.client.constants import DEFAULT_NAMESPACE_NAME
from openlineage.client.run import Job, Run, RunEvent, RunState
from openlineage.client.uuid import generate_new_uuid
from openlineage.dagster.adapter import OpenLineageAdapter

from .conftest import PRODUCER


@patch("openlineage.dagster.adapter.to_utc_iso_8601")
@patch("openlineage.dagster.adapter.OpenLineageClient.emit")
def test_start_pipeline_run(mock_client_emit, mock_to_utc_iso_8601):
    event_time = "2022-01-01T00:00:00.000000Z"
    mock_to_utc_iso_8601.return_value = event_time

    pipeline_name = "a_pipeline"
    pipeline_run_id = str(generate_new_uuid())
    timestamp = time.time()

    adapter = OpenLineageAdapter()
    adapter.start_pipeline(pipeline_name, pipeline_run_id, timestamp)

    mock_to_utc_iso_8601.assert_called_once_with(timestamp)
    mock_client_emit.assert_called_once_with(
        RunEvent(
            eventType=RunState.START,
            eventTime=event_time,
            run=Run(runId=pipeline_run_id, facets={}),
            job=Job(namespace=DEFAULT_NAMESPACE_NAME, name=pipeline_name, facets={}),
            producer=PRODUCER,
            inputs=[],
            outputs=[],
        )
    )


@patch("openlineage.dagster.adapter.to_utc_iso_8601")
@patch("openlineage.dagster.adapter.OpenLineageClient.emit")
def test_complete_pipeline_run(mock_client_emit, mock_to_utc_iso_8601):
    event_time = "2022-01-01T00:00:00.000000Z"
    mock_to_utc_iso_8601.return_value = event_time

    pipeline_name = "a_pipeline"
    pipeline_run_id = str(generate_new_uuid())
    timestamp = time.time()

    adapter = OpenLineageAdapter()
    adapter.complete_pipeline(pipeline_name, pipeline_run_id, timestamp)

    mock_to_utc_iso_8601.assert_called_once_with(timestamp)
    mock_client_emit.assert_called_once_with(
        RunEvent(
            eventType=RunState.COMPLETE,
            eventTime=event_time,
            run=Run(runId=pipeline_run_id, facets={}),
            job=Job(namespace=DEFAULT_NAMESPACE_NAME, name=pipeline_name, facets={}),
            producer=PRODUCER,
            inputs=[],
            outputs=[],
        )
    )


@patch("openlineage.dagster.adapter.to_utc_iso_8601")
@patch("openlineage.dagster.adapter.OpenLineageClient.emit")
def test_fail_pipeline_run(mock_client_emit, mock_to_utc_iso_8601):
    event_time = "2022-01-01T00:00:00.000000Z"
    mock_to_utc_iso_8601.return_value = event_time

    pipeline_name = "a_pipeline"
    pipeline_run_id = str(generate_new_uuid())
    timestamp = time.time()

    adapter = OpenLineageAdapter()
    adapter.fail_pipeline(pipeline_name, pipeline_run_id, timestamp)

    mock_to_utc_iso_8601.assert_called_once_with(timestamp)
    mock_client_emit.assert_called_once_with(
        RunEvent(
            eventType=RunState.FAIL,
            eventTime=event_time,
            run=Run(runId=pipeline_run_id, facets={}),
            job=Job(namespace=DEFAULT_NAMESPACE_NAME, name=pipeline_name, facets={}),
            producer=PRODUCER,
            inputs=[],
            outputs=[],
        )
    )


@patch("openlineage.dagster.adapter.to_utc_iso_8601")
@patch("openlineage.dagster.adapter.OpenLineageClient.emit")
def test_cancel_pipeline_run(mock_client_emit, mock_to_utc_iso_8601):
    event_time = "2022-01-01T00:00:00.000000Z"
    mock_to_utc_iso_8601.return_value = event_time

    pipeline_name = "a_pipeline"
    pipeline_run_id = str(generate_new_uuid())
    timestamp = time.time()

    adapter = OpenLineageAdapter()
    adapter.cancel_pipeline(pipeline_name, pipeline_run_id, timestamp)

    mock_to_utc_iso_8601.assert_called_once_with(timestamp)
    mock_client_emit.assert_called_once_with(
        RunEvent(
            eventType=RunState.ABORT,
            eventTime=event_time,
            run=Run(runId=pipeline_run_id, facets={}),
            job=Job(namespace=DEFAULT_NAMESPACE_NAME, name=pipeline_name, facets={}),
            producer=PRODUCER,
            inputs=[],
            outputs=[],
        )
    )



# Copyright 2018-2024 contributors to the OpenLineage project
# SPDX-License-Identifier: Apache-2.0

import uuid
from unittest.mock import patch

from openlineage.client.uuid import generate_new_uuid
from openlineage.dagster.utils import (
    get_event_log_records,
    get_repository_name,
    make_step_job_name,
    make_step_run_id,
    to_utc_iso_8601,
)

from dagster import EventRecordsFilter
from dagster.core.events import PIPELINE_EVENTS, STEP_EVENTS

from .conftest import make_pipeline_run_with_external_pipeline_origin


def test_to_utc_iso_8601():
    assert to_utc_iso_8601(1640995200) == "2022-01-01T00:00:00.000000Z"


def test_make_step_run_id():
    run_id = make_step_run_id()
    assert uuid.UUID(run_id).version == 7


def test_make_step_job_name():
    pipeline_name = "test_job"
    step_key = "test_graph.test_op"
    assert make_step_job_name(pipeline_name, step_key) == "test_job.test_graph.test_op"


@patch("openlineage.dagster.utils.DagsterInstance")
def test_get_event_log_records(mock_instance):
    last_storage_id = 100
    record_filter_limit = 100
    event_types = PIPELINE_EVENTS | STEP_EVENTS
    get_event_log_records(mock_instance, event_types, last_storage_id, record_filter_limit)
    for event_type in event_types:
        mock_instance.get_event_records.assert_any_call(
            EventRecordsFilter(event_type=event_type, after_cursor=last_storage_id),
            limit=record_filter_limit,
        )


@patch("openlineage.dagster.utils.DagsterInstance")
def test_get_repository_name(mock_instance):
    expected_repo = "test_repo"
    pipeline_run_id = str(generate_new_uuid())
    pipeline_run = make_pipeline_run_with_external_pipeline_origin(expected_repo)
    mock_instance.get_run_by_id.return_value = pipeline_run
    actual_repo = get_repository_name(mock_instance, pipeline_run_id)

    assert expected_repo == actual_repo
    mock_instance.get_run_by_id.assert_called_once_with(pipeline_run_id)



# Copyright 2018-2024 contributors to the OpenLineage project
# SPDX-License-Identifier: Apache-2.0

from datetime import datetime
from typing import Iterable, Optional, Set, Union

from openlineage.client.uuid import generate_new_uuid

from dagster import (  # type: ignore
    DagsterEventType,
    DagsterInstance,
    EventLogRecord,
    EventRecordsFilter,
)

NOMINAL_TIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%fZ"


def to_utc_iso_8601(timestamp: float) -> str:
    return datetime.utcfromtimestamp(timestamp).strftime(NOMINAL_TIME_FORMAT)


def make_step_run_id() -> str:
    return str(generate_new_uuid())


def make_step_job_name(pipeline_name: str, step_key: str) -> str:
    return f"{pipeline_name}.{step_key}"


def get_event_log_records(
    instance: DagsterInstance,
    event_type: Union[DagsterEventType, Set[DagsterEventType]],
    last_storage_id: int,
    record_filter_limit: int,
) -> Iterable[EventLogRecord]:
    """Returns a list of Dagster event log records in ascending order
    from the instance's event log storage.
    :param instance: active instance to get records from
    :param event_type: event type to filter by
    :param last_storage_id: storage id to use as after cursor filter
    :param record_filter_limit: maximum number of event logs to retrieve
    :return: list of Dagster event log records
    """
    event_type_set = event_type if isinstance(event_type, set) else set([event_type])
    event_records = []
    for item in event_type_set:
        event_records += instance.get_event_records(
            EventRecordsFilter(event_type=item, after_cursor=last_storage_id),
            limit=record_filter_limit,
        )
    return sorted(event_records, key=lambda record: record.event_log_entry.timestamp)


def get_repository_name(instance: DagsterInstance, pipeline_run_id: str) -> Optional[str]:
    """Returns an optional repository name
    :param instance: active instance to get the pipeline run
    :param pipeline_run_id: run id to look up its run
    :return: optional repository name
    """
    pipeline_run = instance.get_run_by_id(pipeline_run_id)
    repository_name = None
    if pipeline_run:
        # DagsterRun changed attribute name from `external_pipeline_origin` to `external_job_origin`
        # since 1.3.3
        ext_pipeline_origin = (
            pipeline_run.external_pipeline_origin
            if hasattr(pipeline_run, "external_pipeline_origin")
            else pipeline_run.external_job_origin
        )
        if ext_pipeline_origin:
            ext_repository_origin = ext_pipeline_origin.external_repository_origin
            if ext_repository_origin:
                repository_name = ext_repository_origin.repository_name
    return repository_name



# Copyright 2018-2024 contributors to the OpenLineage project
# SPDX-License-Identifier: Apache-2.0

import json
import os
from unittest.mock import patch

from openlineage.client.uuid import generate_new_uuid
from openlineage.dagster.cursor import OpenLineageCursor, RunningPipeline, RunningStep

from dagster import DagsterEventType, SensorDefinition, build_sensor_context
from dagster.core.test_utils import instance_for_test

from .conftest import make_test_event_log_record


@patch.dict(os.environ, {"OPENLINEAGE_URL": "http://mock-url:5000"})
def test_basic_sensor_def():
    from openlineage.dagster.sensor import openlineage_sensor

    sensor_def = openlineage_sensor()
    assert isinstance(sensor_def, SensorDefinition)
    assert not sensor_def.targets


@patch.dict(os.environ, {"OPENLINEAGE_URL": "http://mock-url:5000"})
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_cursor_update_with_after_storage_id(mock_event_log_records):
    from openlineage.dagster.sensor import openlineage_sensor

    with instance_for_test() as instance:
        context = build_sensor_context(instance=instance, repository_name="hello")
        openlineage_sensor(after_storage_id=100).evaluate_tick(context)

        assert context.cursor == json.dumps({"last_storage_id": 100, "running_pipelines": {}})


@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.make_step_run_id")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_cursor_update_with_successful_run(mock_event_log_records, mock_step_run_id, mock_adapter):
    from openlineage.dagster.sensor import openlineage_sensor

    with instance_for_test() as instance:
        ol_sensor_def = openlineage_sensor(record_filter_limit=1)

        # 1. pipeline start
        pipeline_run_id = str(generate_new_uuid())
        mock_event_log_records.return_value = [
            make_test_event_log_record(DagsterEventType.RUN_START, pipeline_run_id=pipeline_run_id)
        ]
        context = build_sensor_context(instance=instance)
        ol_sensor_def.evaluate_tick(context)
        assert OpenLineageCursor.from_json(context.cursor) == OpenLineageCursor(
            last_storage_id=1,
            running_pipelines={pipeline_run_id: RunningPipeline(running_steps={}, repository_name=None)},
        )

        # 2. step start
        step_run_id = str(generate_new_uuid())
        step_key = "an_op"
        mock_step_run_id.return_value = step_run_id
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.STEP_START,
                pipeline_run_id=pipeline_run_id,
                step_key=step_key,
                storage_id=2,
            )
        ]
        ol_sensor_def.evaluate_tick(context)
        assert OpenLineageCursor.from_json(context.cursor) == OpenLineageCursor(
            last_storage_id=2,
            running_pipelines={
                pipeline_run_id: RunningPipeline(
                    running_steps={
                        step_key: RunningStep(
                            step_run_id=step_run_id,
                            input_datasets=[],
                            output_datasets=[],
                        )
                    },
                    repository_name=None,
                )
            },
        )

        # 3. step success
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.STEP_SUCCESS,
                pipeline_run_id=pipeline_run_id,
                step_key=step_key,
                storage_id=3,
            )
        ]
        ol_sensor_def.evaluate_tick(context)
        assert OpenLineageCursor.from_json(context.cursor) == OpenLineageCursor(
            last_storage_id=3,
            running_pipelines={pipeline_run_id: RunningPipeline(running_steps={}, repository_name=None)},
        )

        # 4. pipeline success
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.RUN_SUCCESS,
                pipeline_run_id=pipeline_run_id,
                storage_id=4,
            )
        ]
        ol_sensor_def.evaluate_tick(context)
        assert OpenLineageCursor.from_json(context.cursor) == OpenLineageCursor(
            last_storage_id=4, running_pipelines={}
        )


@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.make_step_run_id")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_cursor_update_with_failing_run(mock_event_log_records, mock_step_run_id, mock_adapter):
    from openlineage.dagster.sensor import openlineage_sensor

    with instance_for_test() as instance:
        ol_sensor_def = openlineage_sensor(record_filter_limit=1)

        # 1. pipeline start
        pipeline_run_id = str(generate_new_uuid())
        mock_event_log_records.return_value = [
            make_test_event_log_record(DagsterEventType.RUN_START, pipeline_run_id=pipeline_run_id)
        ]
        context = build_sensor_context(instance=instance, cursor=None)
        ol_sensor_def.evaluate_tick(context)
        assert OpenLineageCursor.from_json(context.cursor) == OpenLineageCursor(
            last_storage_id=1,
            running_pipelines={pipeline_run_id: RunningPipeline(running_steps={}, repository_name=None)},
        )

        # 2. step start
        step_run_id = str(generate_new_uuid())
        step_key = "an_op"
        mock_step_run_id.return_value = step_run_id
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.STEP_START,
                pipeline_run_id=pipeline_run_id,
                step_key=step_key,
                storage_id=2,
            )
        ]
        ol_sensor_def.evaluate_tick(context)
        assert OpenLineageCursor.from_json(context.cursor) == OpenLineageCursor(
            last_storage_id=2,
            running_pipelines={
                pipeline_run_id: RunningPipeline(
                    running_steps={
                        step_key: RunningStep(
                            step_run_id=step_run_id,
                            input_datasets=[],
                            output_datasets=[],
                        )
                    },
                )
            },
        )

        # 3. step fail
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.STEP_FAILURE,
                pipeline_run_id=pipeline_run_id,
                step_key=step_key,
                storage_id=3,
            )
        ]
        ol_sensor_def.evaluate_tick(context)
        assert OpenLineageCursor.from_json(context.cursor) == OpenLineageCursor(
            last_storage_id=3,
            running_pipelines={pipeline_run_id: RunningPipeline(running_steps={}, repository_name=None)},
        )

        # 4. pipeline fail
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.RUN_SUCCESS,
                pipeline_run_id=pipeline_run_id,
                storage_id=4,
            )
        ]
        ol_sensor_def.evaluate_tick(context)
        assert OpenLineageCursor.from_json(context.cursor) == OpenLineageCursor(
            last_storage_id=4, running_pipelines={}
        )


@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.make_step_run_id")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_cursor_update_with_exception_raised(mock_event_log_records, mock_step_run_id, mock_adapter):
    from openlineage.dagster.sensor import openlineage_sensor

    with instance_for_test() as instance:
        pipeline_run_id = str(generate_new_uuid())
        step_key = "an_op"
        step_run_id = str(generate_new_uuid())
        mock_step_run_id.return_value = step_run_id
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.STEP_START,
                pipeline_run_id=pipeline_run_id,
                step_key=step_key,
            ),
            make_test_event_log_record(
                DagsterEventType.STEP_SUCCESS,
                pipeline_run_id=pipeline_run_id,
                step_key=step_key,
            ),
        ]
        mock_adapter.complete_step.side_effect = Exception("test!")
        context = build_sensor_context(instance=instance)

        openlineage_sensor(record_filter_limit=2).evaluate_tick(context)

        assert OpenLineageCursor.from_json(context.cursor) == OpenLineageCursor(
            last_storage_id=1,
            running_pipelines={
                pipeline_run_id: RunningPipeline(
                    running_steps={
                        step_key: RunningStep(
                            step_run_id=step_run_id,
                            input_datasets=[],
                            output_datasets=[],
                        )
                    },
                    repository_name=None,
                )
            },
        )



# Copyright 2018-2024 contributors to the OpenLineage project
# SPDX-License-Identifier: Apache-2.0

import time
from unittest.mock import patch

from openlineage.client.constants import DEFAULT_NAMESPACE_NAME
from openlineage.client.facet import ParentRunFacet
from openlineage.client.run import Job, Run, RunEvent, RunState
from openlineage.client.uuid import generate_new_uuid
from openlineage.dagster.adapter import OpenLineageAdapter

from .conftest import PRODUCER


@patch("openlineage.dagster.adapter.to_utc_iso_8601")
@patch("openlineage.dagster.adapter.OpenLineageClient")
def test_start_pipeline(mock_client, mock_to_utc_iso_8601):
    event_time = "2022-01-01T00:00:00.000000Z"
    mock_to_utc_iso_8601.return_value = event_time

    pipeline_name = "a_pipeline"
    pipeline_run_id = str(generate_new_uuid())
    step_run_id = str(generate_new_uuid())
    step_key = "an_op"
    timestamp = time.time()

    adapter = OpenLineageAdapter()
    adapter.start_step(pipeline_name, pipeline_run_id, timestamp, step_run_id, step_key)

    mock_to_utc_iso_8601.assert_called_once_with(timestamp)
    mock_client.return_value.emit.assert_called_once_with(
        RunEvent(
            eventType=RunState.START,
            eventTime=event_time,
            run=Run(
                runId=step_run_id,
                facets={
                    "parent": ParentRunFacet(
                        run={"runId": pipeline_run_id},
                        job={
                            "namespace": DEFAULT_NAMESPACE_NAME,
                            "name": pipeline_name,
                        },
                    )
                },
            ),
            job=Job(
                namespace=DEFAULT_NAMESPACE_NAME,
                name=f"{pipeline_name}.{step_key}",
                facets={},
            ),
            producer=PRODUCER,
            inputs=[],
            outputs=[],
        )
    )


@patch("openlineage.dagster.adapter.to_utc_iso_8601")
@patch("openlineage.dagster.adapter.OpenLineageClient")
def test_complete_step(mock_client, mock_to_utc_iso_8601):
    event_time = "2022-01-01T00:00:00.000000Z"
    mock_to_utc_iso_8601.return_value = event_time

    pipeline_name = "a_pipeline"
    pipeline_run_id = str(generate_new_uuid())
    step_run_id = str(generate_new_uuid())
    step_key = "an_op"
    timestamp = time.time()

    adapter = OpenLineageAdapter()
    adapter.complete_step(pipeline_name, pipeline_run_id, timestamp, step_run_id, step_key)

    mock_to_utc_iso_8601.assert_called_once_with(timestamp)
    mock_client.return_value.emit.assert_called_once_with(
        RunEvent(
            eventType=RunState.COMPLETE,
            eventTime=event_time,
            run=Run(
                runId=step_run_id,
                facets={
                    "parent": ParentRunFacet(
                        run={"runId": pipeline_run_id},
                        job={
                            "namespace": DEFAULT_NAMESPACE_NAME,
                            "name": pipeline_name,
                        },
                    )
                },
            ),
            job=Job(
                namespace=DEFAULT_NAMESPACE_NAME,
                name=f"{pipeline_name}.{step_key}",
                facets={},
            ),
            producer=PRODUCER,
            inputs=[],
            outputs=[],
        )
    )


@patch("openlineage.dagster.adapter.to_utc_iso_8601")
@patch("openlineage.dagster.adapter.OpenLineageClient")
def test_fail_step(mock_client, mock_to_utc_iso_8601):
    event_time = "2022-01-01T00:00:00.000000Z"
    mock_to_utc_iso_8601.return_value = event_time

    pipeline_name = "a_pipeline"
    pipeline_run_id = str(generate_new_uuid())
    step_run_id = str(generate_new_uuid())
    step_key = "an_op"
    timestamp = time.time()

    adapter = OpenLineageAdapter()
    adapter.fail_step(pipeline_name, pipeline_run_id, timestamp, step_run_id, step_key)

    mock_to_utc_iso_8601.assert_called_once_with(timestamp)
    mock_client.return_value.emit.assert_called_once_with(
        RunEvent(
            eventType=RunState.FAIL,
            eventTime=event_time,
            run=Run(
                runId=step_run_id,
                facets={
                    "parent": ParentRunFacet(
                        run={"runId": pipeline_run_id},
                        job={
                            "namespace": DEFAULT_NAMESPACE_NAME,
                            "name": pipeline_name,
                        },
                    )
                },
            ),
            job=Job(
                namespace=DEFAULT_NAMESPACE_NAME,
                name=f"{pipeline_name}.{step_key}",
                facets={},
            ),
            producer=PRODUCER,
            inputs=[],
            outputs=[],
        )
    )



# Copyright 2018-2024 contributors to the OpenLineage project
# SPDX-License-Identifier: Apache-2.0

import os

from openlineage.dagster.adapter import OpenLineageAdapter


def test_custom_client_config():
    os.environ["OPENLINEAGE_URL"] = "http://ol-api:5000"
    os.environ["OPENLINEAGE_API_KEY"] = "api-key"
    adapter = OpenLineageAdapter()
    assert adapter._client.transport.url == "http://ol-api:5000"
    del os.environ["OPENLINEAGE_URL"]
    del os.environ["OPENLINEAGE_API_KEY"]



# Copyright 2018-2024 contributors to the OpenLineage project
# SPDX-License-Identifier: Apache-2.0

import tempfile
import time
from unittest import mock
from unittest.mock import call, patch

from openlineage.client.uuid import generate_new_uuid
from openlineage.dagster.sensor import openlineage_sensor

from dagster import (
    DagsterEventType,
    build_sensor_context,
    execute_job,
    job,
    op,
    reconstructable,
)
from dagster.core.test_utils import instance_for_test

from .conftest import make_test_event_log_record


@job
def a_job():
    @op
    def an_op():
        pass

    an_op()


@patch("openlineage.dagster.sensor.get_repository_name")
@patch("openlineage.dagster.sensor.make_step_run_id")
@patch("openlineage.dagster.sensor._ADAPTER")
def test_sensor_with_complete_job_run_and_repository(
    mock_adapter, mock_step_run_id, mock_get_repository_name
):
    with tempfile.TemporaryDirectory() as temp_dir:
        with instance_for_test(
            temp_dir=temp_dir,
            overrides={  # to avoid run-sharded event log storage warning
                "event_log_storage": {
                    "module": "dagster.core.storage.event_log",
                    "class": "ConsolidatedSqliteEventLogStorage",
                    "config": {"base_dir": temp_dir},
                },
            },
        ) as instance:
            repository_name = "a_repository"
            pipeline_name = "a_job"
            step_key = "an_op"
            step_run_id = str(generate_new_uuid())
            mock_step_run_id.return_value = step_run_id
            mock_get_repository_name.return_value = repository_name

            result = execute_job(job=reconstructable(a_job), instance=instance, raise_on_error=False)
            pipeline_run_id = result.run_id

            context = build_sensor_context(instance=instance)
            previous_cursor = None
            while True:
                openlineage_sensor().evaluate_tick(context)
                if context.cursor == previous_cursor:
                    break
                else:
                    previous_cursor = context.cursor

            mock_adapter.assert_has_calls(
                [
                    call.start_pipeline(pipeline_name, pipeline_run_id, mock.ANY, repository_name),
                    call.start_step(
                        pipeline_name,
                        pipeline_run_id,
                        mock.ANY,
                        step_run_id,
                        step_key,
                        repository_name,
                    ),
                    call.complete_step(
                        pipeline_name,
                        pipeline_run_id,
                        mock.ANY,
                        step_run_id,
                        step_key,
                        repository_name,
                    ),
                    call.complete_pipeline(pipeline_name, pipeline_run_id, mock.ANY, repository_name),
                ]
            )


@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_sensor_start_pipeline(mock_event_log_records, mock_adapter):
    with instance_for_test() as instance:
        pipeline_name = "a_job"
        pipeline_run_id = str(generate_new_uuid())
        timestamp = time.time()
        mock_event_log_records.return_value = [
            make_test_event_log_record(DagsterEventType.RUN_START, pipeline_name, pipeline_run_id, timestamp)
        ]

        context = build_sensor_context(instance=instance)
        openlineage_sensor().evaluate_tick(context)

        mock_adapter.assert_has_calls(
            [mock.call.start_pipeline(pipeline_name, pipeline_run_id, timestamp, None)]
        )


@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_sensor_complete_pipeline(mock_event_log_records, mock_adapter):
    with instance_for_test() as instance:
        pipeline_name = "a_job"
        pipeline_run_id = str(generate_new_uuid())
        timestamp = time.time()
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.RUN_SUCCESS, pipeline_name, pipeline_run_id, timestamp
            )
        ]

        context = build_sensor_context(instance=instance)
        openlineage_sensor().evaluate_tick(context)

        mock_adapter.assert_has_calls(
            [mock.call.complete_pipeline(pipeline_name, pipeline_run_id, timestamp, None)]
        )


@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_sensor_fail_pipeline(mock_event_log_records, mock_adapter):
    with instance_for_test() as instance:
        pipeline_name = "a_job"
        pipeline_run_id = str(generate_new_uuid())
        timestamp = time.time()
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.RUN_FAILURE, pipeline_name, pipeline_run_id, timestamp
            )
        ]

        context = build_sensor_context(instance=instance)
        openlineage_sensor().evaluate_tick(context)

        mock_adapter.assert_has_calls(
            [mock.call.fail_pipeline(pipeline_name, pipeline_run_id, timestamp, None)]
        )


@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_sensor_cancel_pipeline(mock_event_log_records, mock_adapter):
    with instance_for_test() as instance:
        pipeline_name = "a_job"
        pipeline_run_id = str(generate_new_uuid())
        timestamp = time.time()
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.RUN_CANCELED, pipeline_name, pipeline_run_id, timestamp
            )
        ]

        context = build_sensor_context(instance=instance)
        openlineage_sensor().evaluate_tick(context)

        mock_adapter.assert_has_calls(
            [mock.call.cancel_pipeline(pipeline_name, pipeline_run_id, timestamp, None)]
        )


@patch("openlineage.dagster.sensor.make_step_run_id")
@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_sensor_start_step(mock_event_log_records, mock_adapter, mock_new_step_run_id):
    with instance_for_test() as instance:
        pipeline_name = "a_job"
        pipeline_run_id = str(generate_new_uuid())
        timestamp = time.time()
        step_run_id = str(generate_new_uuid())
        step_key = "an_op"
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.STEP_START,
                pipeline_name,
                pipeline_run_id,
                timestamp,
                step_key,
            )
        ]
        mock_new_step_run_id.return_value = step_run_id

        context = build_sensor_context(instance=instance)
        openlineage_sensor().evaluate_tick(context)

        mock_adapter.assert_has_calls(
            [
                mock.call.start_step(
                    pipeline_name,
                    pipeline_run_id,
                    timestamp,
                    step_run_id,
                    step_key,
                    None,
                )
            ]
        )


@patch("openlineage.dagster.sensor.make_step_run_id")
@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_sensor_complete_step(mock_event_log_records, mock_adapter, mock_new_step_run_id):
    with instance_for_test() as instance:
        pipeline_name = "a_job"
        pipeline_run_id = str(generate_new_uuid())
        timestamp = time.time()
        step_run_id = str(generate_new_uuid())
        step_key = "an_op"
        mock_new_step_run_id.return_value = step_run_id
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.STEP_SUCCESS,
                pipeline_name,
                pipeline_run_id,
                timestamp,
                step_key,
            )
        ]

        context = build_sensor_context(instance=instance)
        openlineage_sensor().evaluate_tick(context)

        mock_adapter.assert_has_calls(
            [
                mock.call.complete_step(
                    pipeline_name,
                    pipeline_run_id,
                    timestamp,
                    step_run_id,
                    step_key,
                    None,
                )
            ]
        )


@patch("openlineage.dagster.sensor.make_step_run_id")
@patch("openlineage.dagster.sensor._ADAPTER")
@patch("openlineage.dagster.sensor.get_event_log_records")
def test_sensor_fail_step(mock_event_log_records, mock_adapter, mock_new_step_run_id):
    with instance_for_test() as instance:
        pipeline_name = "a_job"
        pipeline_run_id = str(generate_new_uuid())
        timestamp = time.time()
        step_run_id = str(generate_new_uuid())
        step_key = "an_op"
        mock_event_log_records.return_value = [
            make_test_event_log_record(
                DagsterEventType.STEP_FAILURE,
                pipeline_name,
                pipeline_run_id,
                timestamp,
                step_key,
            )
        ]
        mock_new_step_run_id.return_value = step_run_id

        context = build_sensor_context(instance=instance)
        openlineage_sensor().evaluate_tick(context)

        mock_adapter.assert_has_calls(
            [
                mock.call.fail_step(
                    pipeline_name,
                    pipeline_run_id,
                    timestamp,
                    step_run_id,
                    step_key,
                    None,
                )
            ]
        )



#!/usr/bin/env python
#
# Copyright 2018-2024 contributors to the OpenLineage project
# SPDX-License-Identifier: Apache-2.0
#
# -*- coding: utf-8 -*-
# import setuptools
from setuptools import find_namespace_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()

__version__ = "1.20.5"

DAGSTER_VERSION = "1.0.0"

requirements = [
    "attrs>=19.3",
    "cattrs",
    "protobuf<=3.20.0",
    f"dagster>={DAGSTER_VERSION},<=1.6.9",
    f"openlineage-python=={__version__}",
]

extras_require = {
    "tests": [
        "pytest",
        "pytest-cov",
        "mypy>=0.9.6",
        # The relevant compat layer of pydantic v2 is shipped with Dagster `1.5.5`.
        # https://github.com/dagster-io/dagster/issues/15162
        "pydantic<2.0.0",
        "sqlalchemy<2.0.0",
    ],
}

extras_require["dev"] = extras_require["tests"]

setup(
    name="openlineage-dagster",
    version=__version__,
    description="OpenLineage integration with Dagster",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="OpenLineage",
    packages=find_namespace_packages(include=["openlineage.*"]),
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    python_requires=">=3.8",
    zip_safe=False,
    keywords="openlineage",
)



# Copyright 2018-2024 contributors to the OpenLineage project
# SPDX-License-Identifier: Apache-2.0

import logging
import os
from typing import Dict, Optional

from openlineage.client import OpenLineageClient, set_producer
from openlineage.client.constants import DEFAULT_NAMESPACE_NAME
from openlineage.client.facet import ParentRunFacet
from openlineage.client.run import Job, Run, RunEvent, RunState
from openlineage.dagster import __version__ as OPENLINEAGE_DAGSTER_VERSION
from openlineage.dagster.utils import make_step_job_name, to_utc_iso_8601

_DEFAULT_NAMESPACE_NAME = os.getenv("OPENLINEAGE_NAMESPACE", DEFAULT_NAMESPACE_NAME)
_PRODUCER = (
    f"https://github.com/OpenLineage/OpenLineage/tree/" f"{OPENLINEAGE_DAGSTER_VERSION}/integration/dagster"
)

set_producer(_PRODUCER)

log = logging.getLogger(__name__)


class OpenLineageAdapter:
    """Adapter to translate Dagster metadata to OpenLineage events
    and emit them to OpenLineage backend
    """

    def __init__(self):
        self._client = OpenLineageClient()

    def start_pipeline(
        self,
        pipeline_name: str,
        pipeline_run_id: str,
        timestamp: float,
        repository_name: Optional[str] = None,
    ):
        """Emits OpenLineage event to indicate start of Dagster pipeline run.
        :param pipeline_name: Dagster pipeline name
        :param pipeline_run_id: Dagster-generated unique identifier for a pipeline run
        :param timestamp: Unix timestamp of Dagster event
        :param repository_name: Dagster repository name
        :return:
        """
        self._emit_pipeline_event(RunState.START, pipeline_name, pipeline_run_id, timestamp, repository_name)

    def complete_pipeline(
        self,
        pipeline_name: str,
        pipeline_run_id: str,
        timestamp: float,
        repository_name: Optional[str] = None,
    ):
        """Emits OpenLineage event to indicate completion of Dagster pipeline run.
        :param pipeline_name: Dagster pipeline name
        :param pipeline_run_id: Dagster-generated unique identifier for a pipeline run
        :param timestamp: Unix timestamp of Dagster event
        :param repository_name: Dagster repository name
        :return:
        """
        self._emit_pipeline_event(
            RunState.COMPLETE,
            pipeline_name,
            pipeline_run_id,
            timestamp,
            repository_name,
        )

    def fail_pipeline(
        self,
        pipeline_name: str,
        pipeline_run_id: str,
        timestamp: float,
        repository_name: Optional[str] = None,
    ):
        """Emits OpenLineage event to indicate failure of Dagster pipeline run.
        :param pipeline_name: Dagster pipeline name
        :param pipeline_run_id: Dagster-generated unique identifier for a pipeline run
        :param timestamp: Unix timestamp of Dagster event
        :param repository_name: Dagster repository name
        :return:
        """
        self._emit_pipeline_event(RunState.FAIL, pipeline_name, pipeline_run_id, timestamp, repository_name)

    def cancel_pipeline(
        self,
        pipeline_name: str,
        pipeline_run_id: str,
        timestamp: float,
        repository_name: Optional[str] = None,
    ):
        """Emits OpenLineage event to indicate cancellation of Dagster pipeline run.
        :param pipeline_name: Dagster pipeline name
        :param pipeline_run_id: Dagster-generated unique identifier for a pipeline run
        :param timestamp: Unix timestamp of Dagster event
        :param repository_name: Dagster repository name
        :return:
        """
        self._emit_pipeline_event(RunState.ABORT, pipeline_name, pipeline_run_id, timestamp, repository_name)

    def _emit_pipeline_event(
        self,
        event_type: RunState,
        pipeline_name: str,
        pipeline_run_id: str,
        timestamp: float,
        repository_name: Optional[str],
    ):
        self._emit(
            RunEvent(
                eventType=event_type,
                eventTime=to_utc_iso_8601(timestamp),
                run=self._build_run(
                    namespace=repository_name if repository_name else _DEFAULT_NAMESPACE_NAME,
                    run_id=pipeline_run_id,
                ),
                job=self._build_job(
                    namespace=repository_name if repository_name else _DEFAULT_NAMESPACE_NAME,
                    job_name=pipeline_name,
                ),
                producer=_PRODUCER,
            )
        )

    def start_step(
        self,
        pipeline_name: str,
        pipeline_run_id: str,
        timestamp: float,
        step_run_id: str,
        step_key: str,
        repository_name: Optional[str] = None,
    ):
        """Emits OpenLineage event to indicate start of Dagster step (op) execution.
        :param pipeline_name: Dagster pipeline name
        :param pipeline_run_id: Dagster-generated unique identifier for a pipeline run
        :param timestamp: Unix timestamp of Dagster event
        :param step_run_id: Unique identifier for a step run (not associated with Dagster)
        :param step_key: Dagster step key
        :param repository_name: Dagster repository name
        :return:
        """
        self._emit_step_event(
            RunState.START,
            pipeline_name,
            pipeline_run_id,
            timestamp,
            step_run_id,
            step_key,
            repository_name,
        )

    def complete_step(
        self,
        pipeline_name: str,
        pipeline_run_id: str,
        timestamp: float,
        step_run_id: str,
        step_key: str,
        repository_name: Optional[str] = None,
    ):
        """Emits OpenLineage event to indicate completion of Dagster step (op) execution.
        :param pipeline_name: Dagster pipeline name
        :param pipeline_run_id: Dagster-generated unique identifier for a pipeline run
        :param timestamp: Unix timestamp of Dagster event
        :param step_run_id: Unique identifier for a step run (not associated with Dagster)
        :param step_key: Dagster step key
        :param repository_name: Dagster repository name
        :return:
        """
        self._emit_step_event(
            RunState.COMPLETE,
            pipeline_name,
            pipeline_run_id,
            timestamp,
            step_run_id,
            step_key,
            repository_name,
        )

    def fail_step(
        self,
        pipeline_name: str,
        pipeline_run_id: str,
        timestamp: float,
        step_run_id: str,
        step_key: str,
        repository_name: Optional[str] = None,
    ):
        """Emits OpenLineage event to indicate failure of Dagster step (op) execution.
        :param pipeline_name: Dagster pipeline name
        :param pipeline_run_id: Dagster-generated unique identifier for a pipeline run
        :param timestamp: Unix timestamp of Dagster event
        :param step_run_id: Unique identifier for a step run (not associated with Dagster)
        :param step_key: Dagster step key
        :param repository_name: Dagster repository name
        :return:
        """
        self._emit_step_event(
            RunState.FAIL,
            pipeline_name,
            pipeline_run_id,
            timestamp,
            step_run_id,
            step_key,
            repository_name,
        )

    def _emit_step_event(
        self,
        event_type: RunState,
        pipeline_name: str,
        pipeline_run_id: str,
        timestamp: float,
        step_run_id: str,
        step_key: str,
        repository_name: Optional[str],
    ):
        self._emit(
            RunEvent(
                eventType=event_type,
                eventTime=to_utc_iso_8601(timestamp),
                run=self._build_run(
                    namespace=repository_name if repository_name else _DEFAULT_NAMESPACE_NAME,
                    run_id=step_run_id,
                    parent_run_id=pipeline_run_id,
                    parent_job_name=pipeline_name,
                ),
                job=self._build_job(
                    namespace=repository_name if repository_name else _DEFAULT_NAMESPACE_NAME,
                    job_name=make_step_job_name(pipeline_name, step_key),
                ),
                producer=_PRODUCER,
            )
        )

    def _emit(self, event: RunEvent):
        self._client.emit(event)
        log.debug(f"Successfully emitted OpenLineage run event: {event}")

    @staticmethod
    def _build_run(
        namespace: str,
        run_id: str,
        parent_run_id: Optional[str] = None,
        parent_job_name: Optional[str] = None,
    ) -> Run:
        facets: Dict = {}
        if parent_run_id is not None and parent_job_name is not None:
            facets.update({"parent": ParentRunFacet.create(parent_run_id, namespace, parent_job_name)})
        return Run(run_id, facets)

    @staticmethod
    def _build_job(namespace: str, job_name: str) -> Job:
        facets: Dict = {}
        return Job(namespace, job_name, facets)
